#include <string.h>
#include <math.h>
#include "stdtypes.h"

/****************************************************************************
 Objectives : encode/decode the image frames
 Input      : no. of mcu per lines, no. of mcu lines, quantization factor
	      image frame to be encoded/decoded,
	      data structure for the decoded image frames
 Output     : NONE
 Called By  : encode()
 Calls      : get_block(), put_block(), dct(), idct(), quantize(), iquantize()
****************************************************************************/
void mcu_coding(image, prev, line_cnt, mcu_cnt, quant)
int    line_cnt, mcu_cnt, quant;
FRAME  *image, *prev;
{
	/* locations for 8 X 8 block */
	long	block[8][8], transformed_block[8][8]; 
 	BLOCK   mcu[6];		/* blocks in a mcu (Y:Cr:Cb = 4:1:1) */
	int	u, v, x, y;     /* no. of blocks in a MCU 	     */
	int 	i = 0;
	FILE    *fp;

	u = mcu_cnt << 1;
	v = line_cnt << 1;

	for (y=0; y<2; y++)   /* YFRAME encoding/decoding */
	   for (x=0; x<2; x++) {
	 	/********************/
		/*  Encoding        */
	        /********************/
		/* get a block from the original image frame */
		get_block(x+u, y+v, YFRAME, image->y, block);
		/* forward DCT of the block */
		dct(block, transformed_block);
		/* quantization of DCT block */
		quantize(transformed_block, &mcu[i], quant, YFRAME);

	 	/********************/
		/*  Decoding        */
	        /********************/
		/* inverse quantization of DCT block */
		iquantize(&mcu[i++], block, quant, YFRAME);
		/* IDCT of the block */
		idct(block);
		/* store the block to the decoded frame */
		put_block((*prev).y, block, x+u, y+v, YFRAME);

		/*
		idct(transformed_block);
		put_block(prev->y, transformed_block, x+u, y+v, YFRAME);
		*/
	   }
	   

   	u = mcu_cnt; 
	v = line_cnt;


   	/* VFRAME encoding/decoding */
	/* encoding */
   	get_block(u, v, VFRAME, image->v, block);
	dct(block, transformed_block);
	quantize(transformed_block, &mcu[i], quant, VFRAME);
        /* decoding */
	iquantize(&mcu[i++], block, quant, VFRAME);
   	idct(block);
   	put_block(prev->v, block, u, v, VFRAME);

   	/* UFRAME encoding */
	/* encoding */
   	get_block(u, v, UFRAME, image->u, block);
	dct(block, transformed_block);
	quantize(transformed_block, &mcu[i], quant, UFRAME);
	/* decoding */
	iquantize(&mcu[i++], block, quant, UFRAME);
   	idct(block);
   	put_block(prev->u, block, u, v, UFRAME);

	/* Entropy Coding */
	/*huff_encode(mcu);*/
};

/****************************************************************************
 Objectives : encode the image frames
 Input      : no. of mcu per lines, no. of mcu lines, quantization factor
	      image frame to be encoded/decoded,
	      data structure for the decoded image frames
 Output     : NONE
 Called By  : main()
 Calls      : mcu_coding()
****************************************************************************/
encode(q, line, mcu_per_line, image, prev) 
int    q, line, mcu_per_line;
FRAME  *image, *prev;
{
  int 	  quant = q; /* quantization factor */
  int 	  line_cnt;  /* line counter 	    */
  int 	  mcu_cnt;   /* mcu counter  	    */
	
  for (line_cnt=0; line_cnt<line; line_cnt++) 
  {
    for (mcu_cnt=0; mcu_cnt<mcu_per_line; mcu_cnt++) 
    {
	mcu_coding(image, prev, line_cnt, mcu_cnt, quant); 
    }
  }
};

